"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.scaleBetween = exports.createQuery = exports.createQueryHistoryAutocompleter = void 0;
const compass_components_1 = require("@mongodb-js/compass-components");
const mongodb_query_parser_1 = require("mongodb-query-parser");
const editor_1 = require("../editor");
const highlight_1 = require("@lezer/highlight");
const editor_2 = require("../editor");
const createQueryHistoryAutocompleter = (savedQueries, onApply, theme) => {
    return function queryCompletions(context) {
        if (savedQueries.length === 0) {
            return null;
        }
        const maxTime = savedQueries[savedQueries.length - 1].lastExecuted.getTime();
        const minTime = savedQueries[0].lastExecuted.getTime();
        const options = savedQueries.map((query) => ({
            label: createQuery(query),
            type: query.type === 'recent' ? 'query-history' : 'favorite',
            detail: (0, compass_components_1.formatDate)(query.lastExecuted.getTime()),
            info: () => createInfo(query, theme).dom,
            apply: () => {
                onApply(query.queryProperties);
            },
            boost: scaleBetween(query.lastExecuted.getTime(), -99, 99, minTime, maxTime),
        }));
        return {
            from: context.pos,
            options: options,
        };
    };
};
exports.createQueryHistoryAutocompleter = createQueryHistoryAutocompleter;
const queryLabelStyles = (0, compass_components_1.css)({
    textTransform: 'capitalize',
    fontWeight: 'bold',
    fontFamily: compass_components_1.fontFamilies.default,
});
const queryCodeStyles = (0, compass_components_1.css)({
    fontFamily: compass_components_1.fontFamilies.code,
    margin: `${compass_components_1.spacing[50]}px`,
    marginLeft: `${compass_components_1.spacing[100]}px`,
    padding: 0,
    whiteSpace: 'pre-wrap',
});
function createQuery(query) {
    let res = '';
    Object.entries(query.queryProperties).forEach(([key, value]) => {
        const formattedQuery = (0, mongodb_query_parser_1.toJSString)(value);
        const noFilterKey = key === 'filter' ? '' : `${key}: `;
        res += formattedQuery ? `, ${noFilterKey}${formattedQuery}` : '';
    });
    return res.slice(2, res.length);
}
exports.createQuery = createQuery;
const javascriptExpressionLanguageParser = editor_1.languages['javascript-expression']().language.parser;
function createInfo(query, theme) {
    const customHighlighter = editor_2.highlightStyles[theme];
    const container = document.createElement('div');
    Object.entries(query.queryProperties).forEach(([key, value]) => {
        const formattedQuery = (0, mongodb_query_parser_1.toJSString)(value);
        const codeDiv = document.createElement('div');
        const label = document.createElement('label');
        label.className = queryLabelStyles;
        label.textContent = key;
        const code = document.createElement('pre');
        code.className = queryCodeStyles;
        function emit(text, classes) {
            const node = document.createTextNode(text);
            if (classes) {
                const span = document.createElement('span');
                span.appendChild(node);
                span.className = classes;
                code.appendChild(span);
            }
            else {
                code.appendChild(node);
            }
        }
        function emitBreak() {
            code.appendChild(document.createTextNode('\n'));
        }
        if (formattedQuery) {
            (0, highlight_1.highlightCode)(formattedQuery, javascriptExpressionLanguageParser.parse(formattedQuery), customHighlighter, emit, emitBreak);
        }
        codeDiv.append(label);
        codeDiv.appendChild(code);
        container.appendChild(codeDiv);
    });
    return {
        dom: container,
        destroy: () => {
            while (container.firstChild) {
                container.removeChild(container.firstChild);
            }
        },
    };
}
function scaleBetween(unscaledNum, newScaleMin, newScaleMax, originalScaleMin, originalScaleMax) {
    if (originalScaleMax === originalScaleMin)
        return newScaleMin + (newScaleMax - newScaleMin) / 2;
    return (((newScaleMax - newScaleMin) * (unscaledNum - originalScaleMin)) /
        (originalScaleMax - originalScaleMin) +
        newScaleMin);
}
exports.scaleBetween = scaleBetween;
//# sourceMappingURL=query-history-autocompleter.js.map